from weaviate.collections.classes.data import DataObject, DataReference
from weaviate.collections.classes.types import GeoCoordinate, PhoneNumber

__all__ = [
    "DataObject",
    "DataReference",
    "GeoCoordinate",
    "PhoneNumber",
]
